part of katana_auth;

abstract class AuthAdapter {
  bool get enabled;

  Future<void> tryRestoreAuth();

  /// Guest login.
  Future<void> signInAnonymously();

  /// Login using your [email] and [password].
  Future<void> signInEmailAndPassword({
    required String email,
    required String password,
    DynamicMap? data,
    String userPath = "user",
  });

  /// Log out.
  Future<void> signOut();

  /// Register as a user using your [email] and [password].
  Future<void> registerInEmailAndPassword({
    required String email,
    required String password,
    DynamicMap? data,
    String userPath = "user",
  });

  /// Link by email link.
  ///
  /// You need to do [sendEmailLink] first.
  ///
  /// Enter the link acquired by Dynamic Link.
  Future<void> signInEmailLink({required String link});

  /// Send an email link.
  Future<void> sendEmailLink({
    required String email,
    required String url,
    required String packageName,
    int androidMinimumVersion = 1,
  });

  /// Authenticate by sending a code to your phone number.
  Future<void> sendSMS({required String phoneNumber});

  /// Authenticate by sending a code to your phone number.
  Future<void> signInSMS({required String smsCode});

  /// Update your phone number.
  /// You need to send an SMS with [sendSMS] in advance.
  Future<void> changePhoneNumber({required String smsCode});

  /// Resend the email for email address verification.
  Future<void> sendEmailVerification();

  /// Email for password reset will be sent to the specified [email].
  Future<void> sendPasswordResetEmail({required String email});

  /// Send you an email to reset your password.
  Future<void> confirmPasswordReset({
    required String code,
    required String password,
  });

  bool get requiredReauthInEmailAndPassword;

  /// Enter your [password] to re-authenticate.
  Future<void> reauthInEmailAndPassword({required String password});

  /// Account delete.
  Future<void> deleteAccount();

  /// Change your email address.
  ///
  /// It is necessary to execute [reauthInEmailAndPassword]
  /// in advance to re-authenticate.
  Future<void> changeEmail({
    required String email,
  });

  /// Change your password.
  ///
  /// It is necessary to execute [reauthInEmailAndPassword]
  /// in advance to re-authenticate.
  Future<void> changePassword({
    required String password,
  });

  /// True if you are signed in.
  bool get isSignedIn;

  /// You can get the UID after authentication is completed.
  ///
  /// Null is returned if authentication is not completed.
  String get userId;

  /// You can get the Email after authentication is completed.
  ///
  /// Null is returned if authentication is not completed.
  String get email;

  /// You can get the status that user email is verified
  /// after authentication is completed.
  bool get isVerified;

  /// You can get the PhoneNumber after authentication is completed.
  ///
  /// Null is returned if authentication is not completed.
  String get phoneNumber;

  /// You can get the PhotoURL after authentication is completed.
  ///
  /// Null is returned if authentication is not completed.
  String get photoURL;

  /// You can get the Display Name after authentication is completed.
  ///
  /// Null is returned if authentication is not completed.
  String get name;

  /// For anonymous logged in users, True.
  bool get isAnonymously;

  /// Returns the ID of the currently active sign-in provider.
  List<String> get activeSignInProviders;
}
